#!/usr/bin/perl

use strict;
use Test::More tests => 321;

####
#### Basic tests: 3
####

# 1
BEGIN { use_ok('FDN::Sql::Basic'); }

# 2
BEGIN { use_ok('FDN::Sql::DBC'); }

# 3
$FDN::Common::Config::CONFIG_ROOT = './t/';
my $dbh = do_connect('test');
is ( ref($dbh), 'HASH', 'do_connect' );
# Before: 0 + (/3) -> 3

####
#### Re-ecriture variables: 13
####

# /1
my $txt = 'Il dit "non" au referendum';
my $res = dochar($txt);
my $exp = '"Il dit \"non\" au referendum"';
is ( $res, $exp, 'dochar');

# /2
$res = dolike($txt);
$exp = '"%Il dit \"non\" au referendum%"';
is ( $res, $exp, 'dolike');

# /3
$res = dodate(undef);
$exp = "''";
is ( $res, $exp, 'dodate');

# /4
$res = dodate('NOW');
$exp = 'now()';
is ( $res, $exp, 'dodate');

# /5
$res = dodate(20050531231213);
$exp = "'20050531231213'";
is ( $res, $exp, 'dodate');

# /6
$txt = 'null';
$res = do_null_or_char($txt);
$exp = 'NULL';
is ( $res, $exp, 'do_null_or_char');

# /7
$res = do_null_or_char(undef);
is ( $res, $exp, 'do_null_or_char');

# /8
$res = do_null_or_char('toto');
$exp = '"toto"';
is ( $res, $exp, 'do_null_or_char');

# /9
$res = dobool(0);
$exp = "'0'";
is ( $res, $exp, 'dobool' );

# /10
$res = dobool(1);
$exp = "'1'";
is ( $res, $exp, 'dobool' );

# /11
$res = dobool(12);
is ( $res, $exp, 'dobool' );

# /12
$res = dobool('coucou');
is ( $res, $exp, 'dobool' );

# /13
$res = dobool(-1);
$exp = "'-1'";
is ( $res, $exp, 'dobool' );
# Before: 3 + (/13) -> 16

####
#### Re-ecriture resultats: 34
####

# /1
my $tmp = {
	'count_rows' => 2,
	'err_db' => 0,
	'rows' => [ { x => 1, y => 2 } ,
		    { x => 3, y => 4 }
	]
};
my $all = \&FDN::Sql::Basic::__return_all_or_one;
my $col = \&FDN::Sql::Basic::__return_all_or_one_col;
$res = &$all($tmp);
is ( ref($res), 'HASH', '__return_all_or_one' );

# /2
is ( $res->{x}, 1, '__return_all_or_one' );

# /3
is ( $res->{y}, 2, '__return_all_or_one' );

# /4
$res = &$col($tmp,'x');
is ( $res, 1, '__return_all_or_one_col');

# /5
$res = &$col($tmp,'y');
is ( $res, 2, '__return_all_or_one_col');

# /6
$res = &$col($tmp,'z');
is ( $res, undef, '__return_all_or_one_col');

# /7
$res = [ &$all($tmp) ];
is ( ref($res), 'ARRAY', '__return_all_or_one' );

# /8
is ( $#$res, 1, '__return_all_or_one' );

# /9
is_deeply( $res, [ { x => 1, y => 2 } , { x => 3, y => 4 } ], '__return_all_or_one' );

# /10
$res = [ &$col($tmp,'x') ];
is_deeply( $res, [ 1, 3 ], '__return_all_or_one_col' );

# /11
$res = [ &$col($tmp,'y') ];
is_deeply( $res, [ 2, 4 ], '__return_all_or_one_col' );

# /12
$res = [ &$col($tmp,'z') ];
is_deeply( $res, [ undef, undef ], '__return_all_or_one_col' );

# /13
$tmp->{'err_db'} = 42;
$res = &$all($tmp);
is ( $res, undef, '__return_all_or_one');

# /14
$res = &$col($tmp,'x');
is ( $res, undef, '__return_all_or_one_col');

# /15
$res = [ &$all($tmp) ];
is_deeply ( $res, [], '__return_all_or_one');

# /16
$res = [ &$col($tmp,'x') ];
is_deeply ( $res, [], '__return_all_or_one_col');

# /17
my $test = \&FDN::Sql::Basic::__rewrite_condition;
$res = &$test( [ 'A', '=', 12 ], 'TBL' );
$exp = 'TBL_A = 12';
is ( $res, $exp, '__rewrite_condition');

# /18
$res = &$test( [ 'OTH_A', '=', 12 ], 'TBL' );
$exp = 'OTH_A = 12';
is ( $res, $exp, '__rewrite_condition');

# /19
$res = &$test( [ 'THG.OTH_A', '=', 12 ], 'TBL' );
$exp = 'THG.OTH_A = 12';
is ( $res, $exp, '__rewrite_condition');

# /20
$res = &$test( [ 'A', 'eq', 12 ], 'TBL' );
$exp = 'TBL_A = 12';
is ( $res, $exp, '__rewrite_condition');

# /21
$res = &$test( [ 'A', '<', 12 ], 'TBL' );
$exp = 'TBL_A < 12';
is ( $res, $exp, '__rewrite_condition');

# /22
$res = &$test( [ 'A', '>', 12 ], 'TBL' );
$exp = 'TBL_A > 12';
is ( $res, $exp, '__rewrite_condition');

# /23
$res = &$test( [ 'A', '<=', 12 ], 'TBL' );
$exp = 'TBL_A <= 12';
is ( $res, $exp, '__rewrite_condition');

# /24
$res = &$test( [ 'A', '>=', 12 ], 'TBL' );
$exp = 'TBL_A >= 12';
is ( $res, $exp, '__rewrite_condition');

# /25
$res = &$test( [ 'A', '!=', 12 ], 'TBL' );
$exp = 'TBL_A != 12';
is ( $res, $exp, '__rewrite_condition');

# /26
$res = &$test( [ 'A', '~', 12 ], 'TBL' );
$exp = 'TBL_A ~ 12';
is ( $res, $exp, '__rewrite_condition');

# /27
$res = &$test( [ 'A', '!~', 12 ], 'TBL' );
$exp = 'TBL_A !~ 12';
is ( $res, $exp, '__rewrite_condition');

# /28
$res = &$test( [ 'A', 'gt', 12 ], 'TBL' );
$exp = 'TBL_A > 12';
is ( $res, $exp, '__rewrite_condition');

# /29
$res = &$test( [ 'A', 'lt', 12 ], 'TBL' );
$exp = 'TBL_A < 12';
is ( $res, $exp, '__rewrite_condition');

# /30
$res = &$test( [ 'A', 'ge', 12 ], 'TBL' );
$exp = 'TBL_A >= 12';
is ( $res, $exp, '__rewrite_condition');

# /31
$res = &$test( [ 'A', 'le', 12 ], 'TBL' );
$exp = 'TBL_A <= 12';
is ( $res, $exp, '__rewrite_condition');

# /32
$res = &$test( [ 'A', 'ne', 12 ], 'TBL' );
$exp = 'TBL_A != 12';
is ( $res, $exp, '__rewrite_condition');

# /33
$res = &$test( [ 'A', 'IS NULL', 12 ], 'TBL' );
$exp = 'TBL_A IS NULL';
is ( $res, $exp, '__rewrite_condition');

# /34
$res = &$test( [ 'A', 'IS NOT NULL', 12 ], 'TBL' );
$exp = 'TBL_A IS NOT NULL';
is ( $res, $exp, '__rewrite_condition');
# Before 16 + (/34) -> 50


####
#### Init db -> 38
####

my $sql = \&FDN::Sql::DB_Access::sql_transaction;
my @queries = (
# /1
"drop table if exists SEQ",

# /2
"create table SEQ (                                        ".
"    SEQ_NAME varchar(20) UNIQUE NOT NULL default ''      ,".
"    SEQ_VALUE int(11) NOT NULL default '0'               ,".
"    SEQ_MINVALUE int(11) NOT NULL default '0'            ,".
"    SEQ_MAXVALUE int(11) NOT NULL default '0'            ,".
"    SEQ_INC int(11) NOT NULL default '0'                 ,".
"    SEQ_CYCLE ENUM('-1','0','1') NOT NULL default '-1'   ,".
"    primary key  (SEQ_NAME))",

# /3
"drop table if exists TEST",

# /4
"create table TEST (                                       ".
"    TEST_ID int(11) UNIQUE NOT NULL default '0'          ,".
"    TEST_NOM varchar(70)                                 ,".
"    TEST_ETAT enum('oui','non','autre','encore')         ,".
"    TEST_DTMOD timestamp(14)                             ,".
"    TEST_DTIN  timestamp(14)                             ,".
"    TEST_DTOUT timestamp(14)                             ,".
"    primary key(TEST_ID)                                 ,".
"    index(TEST_NOM)                                      ,".
"    index(TEST_ETAT) )",

# /5
"replace into SEQ values('TEST_ID',0,0,99999999,1,'0')",

# /6
"drop table if exists TESTSELF",

# /7
"create table TESTSELF (                                   ".
"    TEST_ID int(11) NOT NULL default '0'                 ,".
"    SELF_ID int(11) NOT NULL default '0'                 ,".
"    TESTSELF_NOM varchar(70) )",

# /8
"drop table if exists PROPTEST",

# /9
"create table PROPTEST (                                   ".
"    PROPTEST_ID int(11) UNIQUE NOT NULL default '0'      ,".
"    TEST_ID int(11) default NULL                         ,".
"    PROPTEST_NOM varchar(70) default NULL                ,".
"    PROPTEST_VAL text                                    ,".
"    PROPTEST_PAR int(11) default NULL                    ,".
"    primary key(PROPTEST_ID)                             ,".
"    index(TEST_ID) )",

# /10
"replace into SEQ values('PROPTEST_ID',0,0,99999999,1,'0')",

# /11
"drop table if exists A",

# /12
"create table A (                                          ".
"    A_ID int(11) UNIQUE NOT NULL default '0'             ,".
"    A_NOM varchar(70)                                    ,".
"    primary key(A_ID) )",

# /13
"replace into SEQ values('A_ID',0,0,99999999,1,'0')",

# /14
"drop table if exists B",

# /15
"create table B (                                          ".
"    B_ID int(11) UNIQUE NOT NULL default '0'             ,".
"    B_NOM varchar(70)                                    ,".
"    primary key(B_ID) )",

# /16
"replace into SEQ values('B_ID',0,0,99999999,1,'0')",

# /17
"drop table if exists AB",

# /18
"create table AB (                                         ".
"    A_ID int(11) NOT NULL default '0'                    ,".
"    B_ID int(11) NOT NULL default '0'                    ,".
"    AB_NOM varchar(70)                                   ,".
"    index(A_ID)                                          ,".
"    index(B_ID)                                          ,".
"    index(A_ID,B_ID) )",

# /19
"drop table if exists C",

# /20
"create table C (                                          ".
"    C_ID int(11) UNIQUE NOT NULL default '0'             ,".
"    C_NOM varchar(70)                                    ,".
"    primary key(C_ID) )",

# /21
"replace into SEQ values('C_ID',0,0,99999999,1,'0')",

# /22
"drop table if exists BC",

# /23
"create table BC (                                         ".
"    B_ID int(11) NOT NULL default '0'                    ,".
"    C_ID int(11) NOT NULL default '0'                    ,".
"    index(B_ID)                                          ,".
"    index(C_ID)                                          ,".
"    index(B_ID,C_ID) )",

# /24
"drop table if exists OTHER",

# /25
"create table OTHER (                                      ".
"    OTHER_ID int(11) UNIQUE NOT NULL default '0'         ,".
"    OTHER_NOM varchar(70)                                ,".
"    OTHER_FLD varchar(70)                                ,".
"    TEST_ID int(11) NOT NULL default '0'                 ,".
"    primary key(OTHER_ID)                                ,".
"    index(TEST_ID) )",

# /26
"replace into SEQ values('OTHER_ID',0,0,99999999,1,'0')",

# /27
"drop table if exists ROTATION",

# /28
"create table ROTATION (                                   ".
"    ROTATION_ID int(11) UNIQUE NOT NULL default '0'      ,".
"    ROTATION_TABLE varchar(70)                           ,".
"    ROTATION_DATE  timestamp(14)                         ,".
"    ROTATION_DEST  varchar(70)                           ,".
"    primary key(ROTATION_ID)                             ,".
"    index(ROTATION_TABLE)                                ,".
"    index(ROTATION_DATE) )",

# /29
"replace into SEQ values('ROTATION_ID',0,0,99999999,1,'0')",

# /30
"drop table if exists LOG",

# /31
"drop table if exists LOG20050501020000",

# /32
"drop table if exists LOG20050601020000",

# /33
"drop table if exists LOG20050701020000",

# /34
"create table LOG (                                        ".
"    LOG_ID int(11) UNIQUE NOT NULL default '0'           ,".
"    LOG_DATE timestamp(14)                               ,".
"    LOG_INFO varchar(70)                                 ,".
"    A_ID int(11) NOT NULL default '-1'                   ,".
"    primary key(LOG_ID)                                  ,".
"    index(LOG_DATE) )",

# /35
"replace into SEQ values('LOG_ID',0,0,99999999,1,'0')",

# /36
"drop table if exists NOLOG",

# /37
"create table NOLOG (                                        ".
"    NOLOG_ID int(11) UNIQUE NOT NULL default '0'           ,".
"    NOLOG_DATE timestamp(14)                               ,".
"    NOLOG_INFO varchar(70)                                 ,".
"    primary key(NOLOG_ID)                                  ,".
"    index(NOLOG_DATE) )",

# /38
"replace into SEQ values('NOLOG_ID',0,0,99999999,1,'0')",

);

for my $q ( @queries ) {
	$res = &$sql($dbh,'create',$q);
	ok ( $res->{'ok'}, 'init db' );
}
# Before 50 + ( /38 ) -> 88

####
#### Insertions: 27
####

# /1
$res = get_next($dbh,'A_ID');
is ( $res, 1, 'get_next');

# /2
$res = get_next($dbh,'A_ID');
is ( $res, 2, 'get_next');

# /3
$res = __new($dbh, 'A', 'A_NOM='.dochar('toto'));
is ( $res, 3, '__new');

# /4
$res = __new($dbh, 'A', 'A_NOM='.dochar('titi'));
is ( $res, 4, '__new');

# /5
$res = __new($dbh, 'A', 'A_NOM='.dochar('titi'));
is ( $res, 5, '__new');

# /6
$res = __new($dbh, 'B', 'B_NOM='.dochar('nom1'));
is ( $res, 1, '__new');

# /7
$res = __new($dbh, 'B', 'B_NOM='.dochar('nom2'));
is ( $res, 2, '__new');

# /8
$res = __new($dbh, 'TEST', 'TEST_NOM='.dochar('test1'), 'TEST_ETAT='.dochar('oui'),
		'TEST_DTIN='.dodate('now') );
is ( $res, 1, '__new');

# /9
$res = __new($dbh, 'TEST', 'TEST_NOM='.dochar('test2'), 'TEST_ETAT='.dochar('non'),
		'TEST_DTIN='.dodate('now') );
is ( $res, 2, '__new');

# /10
$res = __new($dbh, 'OTHER', 'OTHER_NOM='.dochar('autre1'), 'OTHER_FLD='.dochar('val1'),
		'TEST_ID=2' );
is ( $res, 1, '__new');

# /11
$res = __new($dbh, 'OTHER', 'OTHER_NOM='.dochar('autre2'), 'OTHER_FLD='.dochar('val2'),
		'TEST_ID=2' );
is ( $res, 2, '__new');

# /12
$res = __new($dbh, 'OTHER', 'OTHER_NOM='.dochar('autre3'), 'OTHER_FLD='.dochar('val3'),
		'TEST_ID=1' );
is ( $res, 3, '__new');

# /13
$res = __set_self_relation($dbh, 'TEST', 1, 2, 'NOM='.dochar('selfrel') );
ok ( $res, '__set_self_relation' );

# /14
$res = __set_relation($dbh, 'A', 'B', 3, 1, 'NOM='.dochar('type1') );
ok ( $res, '__set_relation' );

# /15
$res = __set_relation($dbh, 'A', 'B', 3, 1, 'NOM='.dochar('type2') );
ok ( $res, '__set_relation' );

# /16
$res = __set_relation($dbh, 'A', 'B', 3, 2, 'NOM='.dochar('type1') );
ok ( $res, '__set_relation' );

# /17
$res = __set_relation($dbh, 'A', 'B', 4, 2, 'NOM='.dochar('type1') );
ok ( $res, '__set_relation' );

# /18
$res = __new($dbh, 'TEST', 'TEST_NOM='.dochar('test3'), 'TEST_ETAT='.dochar('non'),
		'TEST_DTIN='.dodate('now') );
is ( $res, 3, '__new');

# /19
$res = __set_self_relation($dbh, 'TEST', 1, 3, 'NOM='.dochar('selfrel2') );
ok ( $res, '__set_self_relation' );

# /20
$res = __new($dbh, 'C', 'NOM='.dochar('elemC-gen-1') );
is ( $res, 1, '__new');

# /21
$res = __new($dbh, 'C', 'NOM='.dochar('elemC-spe-1') );
is ( $res, 2, '__new');

# /22
$res = __new($dbh, 'C', 'NOM='.dochar('elemC-spe-2') );
is ( $res, 3, '__new');

# /23
$res = __new($dbh, 'C', 'NOM='.dochar('elemC-spe-3') );
is ( $res, 4, '__new');

# /24
$res = __set_relation($dbh, 'B', 'C', 1, 1 );
ok ( $res, '__set_relation');

# /25
$res = __set_relation($dbh, 'B', 'C', 1, 2 );
ok ( $res, '__set_relation');

# /26
$res = __set_relation($dbh, 'B', 'C', 2, 1 );
ok ( $res, '__set_relation');

# /27
$res = __set_relation($dbh, 'B', 'C', 2, 3 );
ok ( $res, '__set_relation');
# Before: 88 + (/27) -> 115


####
#### Recherches simples: 38
####

# /1
$res = __par_id($dbh, 'A', 1);
is ( $res, undef, '__par_id' );

# /2
$res = __par_id($dbh, 'A', 2);
is ( $res, undef, '__par_id' );

# /3
$res = __par_id($dbh, 'TEST', 2);
is ( ref($res), 'HASH', '__par_id');

# /4
is ( $res->{'TEST_NOM'}, 'test2', '__par_id');

# /5
is ( $res->{'TEST_ETAT'}, 'non', '__par_id');

# /6
is ( $res->{'TEST_DTIN'}, $res->{'TEST_DTMOD'}, '__par_id' );

# /7
$res = __id_par_nom($dbh, 'A', 'toto');
is ( $res, 3, '__id_par_nom' );

# /8
$res = [ __id_par_nom($dbh, 'A', 'titi' ) ];
ok ( eq_set( $res, [4,5] ), '__id_par_nom' );

# /9
$res = __id_par_clef($dbh, 'OTHER', 1, 'TEST_ID');
is ( $res, 3, '__id_par_clef' );

# /10
$res = __id_par_clef($dbh, 'OTHER', dochar('autre2'), 'OTHER_NOM');
is ( $res, 2, '__id_par_clef' );

# /11
$res = [ __id_par_clef($dbh, 'OTHER', 2, 'TEST_ID') ];
is ( $#$res, 1, '__id_par_clef' );

# /12
ok ( eq_set($res, [1,2]), '__id_par_clef');

# /13
$res = __par_nom_like($dbh,'OTHER','autre1');
is ( ref($res), 'HASH', '__par_nom_like' );

# /14
is ( $res->{'OTHER_ID'}, 1, '__par_nom_like' );

# /15
is ( $res->{'OTHER_NOM'}, 'autre1', '__par_nom_like' );

# /16
is ( $res->{'OTHER_FLD'}, 'val1', '__par_nom_like' );

# /17
is ( $res->{'TEST_ID'}, 2, '__par_nom_like' );

# /18
$res = __par_nom_like($dbh,'OTHER','tre1');
is ( ref($res), 'HASH', '__par_nom_like');

# /19
is ( $res->{'OTHER_ID'}, 1, '__par_nom_like' );

# /20
$res = [ __par_nom_like($dbh,'A', 'i') ];
$exp = [ { 'A_ID' => 4, 'A_NOM' => 'titi' },
	 { 'A_ID' => 5, 'A_NOM' => 'titi' } ];
is ( $#$res, 1, '__par_nom_like' );

# /21
ok ( eq_set($res,$exp), '__par_nom_like' );

# /22
$res = __par_nom_like($dbh,'OTHER','l1','OTHER_FLD');
is ( $res->{'OTHER_ID'}, 1, '__par_nom_like' );

# /23
$res = __par_nom($dbh,'A','toto');
is ( $res->{'A_ID'}, 3, '__par_nom' );

# /24
$res = [ __par_nom($dbh, 'A', 'toto' ) ];
is ( $#$res, 0, '__par_nom' );

# /25
$res = __par_nom($dbh,'A','');
is ( $res, undef, '__par_nom');

# /26
$res = [ __par_nom($dbh, 'A', 'titi' ) ];
ok ( eq_set($res,$exp), '__par_nom' );

# /27
$res = __par_nom($dbh,'OTHER','val3','OTHER_FLD');
is ( $res->{'OTHER_ID'}, 3, '__par_nom');

# /28
$res = __par_clef($dbh,'OTHER','val2','OTHER_FLD');
ok ( defined $res, '__par_clef' );

# /29
is ( ref($res), 'HASH', '__par_clef' );

# /30
is ( $res->{'OTHER_ID'}, 2, '__par_clef' );

# /31
$res = __par_clef($dbh,'OTHER',2,'TEST_ID','orderup(OTHER_ID)');
is ( $res->{'OTHER_ID'}, 1, '__par_clef' );

# /32
$res = __par_clef($dbh,'OTHER',2,'TEST_ID','orderdn(OTHER_ID)');
is ( $res->{'OTHER_ID'}, 2, '__par_clef' );

# /33
$res = __max($dbh, 'TEST', 'TEST_ID');
is ( $res, 3, '__max' );

# /34
$res = __min($dbh, 'TEST', 'TEST_ID');
is ( $res, 1, '__min' );

# /35
$res = __champ_par_id($dbh, 'TEST', 'TEST_ETAT', 2);
is ( $res, 'non', '__champ_par_id');

# /36
$res = __champ_par_id($dbh,'OTHER','TEST_ID', 3);
is ( $res, 1, '__champ_par_id');

# /37
$exp = __par_id($dbh,'TEST',1);
$exp->{'TEST_NOM'} = 'prout';
delete $exp->{'TEST_DTMOD'};
$res = __set_tbl_val($dbh,'TEST','TEST_NOM',1,dochar('prout'));
ok ( $res, '__set_tbl_val');

# /38
$res = __par_id($dbh,'TEST',1);
delete $res->{'TEST_DTMOD'};
is_deeply($res, $exp, '__set_tbl_val');

# Before: 115 + (/38) -> 153


####
#### Relation -> 32
####

# /1-4
$res = __get_relation($dbh,'A','B','A',3,'NOM='.dochar('type2'));
is ( ref($res), 'HASH', '__get_relation' );
is ( $res->{'A_ID'}, 3, '__get_relation' );
is ( $res->{'B_ID'}, 1, '__get_relation' );
is ( $res->{'AB_NOM'}, 'type2', '__get_relation');

# /5
$res = [ __get_relation($dbh,'A','B','A',3,'NOM='.dochar('type2')) ];
is ( $#$res, 0, '__get_relation' );

# /6
$res = [ __get_relation($dbh,'A','B','A',3) ];
is ( $#$res, 2, '__get_relation' );

# /7
$res = [ __get_relation($dbh,'A','B','B',2) ];
is ( $#$res, 1, '__get_relation' );

# /8-14
$res = [ __get_relation_ordonnee($dbh,'A','B','A',3,['orderup(AB_NOM)', 'orderup(B_ID)'] ) ];
is ( $#$res, 2, '__get_relation' );
is ( $res->[0]->{'B_ID'},    1,      '__get_relation_ordonnee' );
is ( $res->[0]->{'AB_NOM'}, 'type1', '__get_relation_ordonnee' );
is ( $res->[1]->{'B_ID'},    2,      '__get_relation_ordonnee' );
is ( $res->[1]->{'AB_NOM'}, 'type1', '__get_relation_ordonnee' );
is ( $res->[2]->{'B_ID'},    1,      '__get_relation_ordonnee' );
is ( $res->[2]->{'AB_NOM'}, 'type2', '__get_relation_ordonnee' );

# /15-17
$res = [ __get_relation_ordonnee($dbh,'A','B','B',2,['orderup(A_ID)']) ];
is ( $#$res, 1, '__get_relation' );
is ( $res->[0]->{'A_ID'}, 3, '__get_relation_ordonnee' );
is ( $res->[1]->{'A_ID'}, 4, '__get_relation_ordonnee' );

# /18
$res = [ __get_relation_ordonnee($dbh,'A','B','B',2) ];
is ( $#$res, 1, '__get_relation' );

# /19-22
$res = __get_self_relation($dbh,'TEST','TEST',1,'NOM='.dochar('selfrel'));
is ( ref($res), 'HASH', '__get_self_relation' );
is ( $res->{'TEST_ID'}, 1, '__get_self_relation' );
is ( $res->{'SELF_ID'}, 2, '__get_self_relation' );
is ( $res->{'TESTSELF_NOM'}, 'selfrel', '__get_self_relation' );

# /23
$res = [ __get_self_relation($dbh,'TEST','TEST',1) ];
is ( $#$res, 1, '__get_self_relation');

# /24-26
$res = [ __get_self_relation_ordonnee($dbh,'TEST','TEST',1,'orderup(SELF_ID)') ];
is ( $#$res, 1, '__get_self_relation_ordonnee');
is ( $res->[0]->{'SELF_ID'}, 2, '__get_self_relation_ordonnee' );
is ( $res->[1]->{'SELF_ID'}, 3, '__get_self_relation_ordonnee' );

# /27
$res = [ __get_self_relation_ordonnee($dbh,'TEST','TEST',1) ];
is ( $#$res, 1, '__get_self_relation_ordonnee');

# /28
$res = [ __par_multi_rel($dbh,[ [ 'B_NOM', '=', dochar('nom2') ] ],
			undef, [ 'A','B' ], [ 'B','C'] ) ];
is ( $#$res, 3, '__par_multi_rel' );

# /29
$res = [ __par_multi_rel($dbh,[ [ 'B_NOM', '=', dochar('nom2') ] ],
			undef, [ 'A','B' ], [ undef,'C'] ) ];
is ( $#$res, 3, '__par_multi_rel' );

# /30-31
$res = __jointure($dbh,[['TEST','OTHER']],undef,{'order' => 'orderup(TEST.TEST_ID)'});
is ( ref($res), 'HASH', '__jointure' );
is ( $res->{'TEST_ID'}, 1, '__jointure');

# /32
$res = [ __jointure($dbh,[['TEST','OTHER']],undef,{'order' => 'orderup(TEST.TEST_ID)'}) ];
is ( $#$res, 2, '__jointure');
# Before: 153 + (/32) -> 185

####
#### Properties -> 10
####

my $props = {
	'un' => 1,
	'deux' => { 'clef1' => 'val1', 'clef2' => 'val2' },
	'trois' => [ 1, 2, 3 ],
	'quatre' => [ { 'clef1' => 'val1' }, { 'clef2' => 'val2' } ]
};

# /1-8
__set_properties($dbh, 'TEST', 1, $props);
$res = __get_properties($dbh, 'TEST', 1);
is ( ref($res), 'HASH', '__get_properties' );
is ( $res->{'un'}, 1,   '__get_properties' );
is ( ref($res->{'deux'}), 'HASH', '__get_properties' );
is ( ref($res->{'trois'}), 'ARRAY', '__get_properties');
is ( ref($res->{'quatre'}), 'ARRAY', '__get_properties');
ok ( eq_set ( $res->{'trois'}, $props->{'trois'} ), '__get_properties' );
ok ( eq_set ( $res->{'quatre'}, $props->{'quatre'} ), '__get_properties' );
is_deeply ( $res->{'deux'}, $props->{'deux'}, '__get_properties' );

# /9
__del_properties($dbh,'TEST',1);
$res = __get_properties($dbh, 'TEST', 1);
is_deeply ( $res, {}, '__del_properties' );

# /10
pop @{$props->{'trois'}};
__update_properties($dbh,'TEST',1,$props);
$res = __get_properties($dbh, 'TEST', 1);
ok ( eq_set ( $res->{'trois'}, $props->{'trois'} ), '__update_properties' );
# Before: 185 + (/10) -> 195

####
#### Suppressions: 16
####

# /1
$res = __par_id($dbh, 'TEST', 3);
is ( ref($res), 'HASH', '__del');

# /2
is ( $res->{'TEST_ID'}, 3, '__del');

# /3
__del($dbh, 'TEST', 3);
$res = __par_id($dbh, 'TEST', 3);
is ( $res, undef, '__del');

# /4
$res = __new($dbh,'TEST','TEST_NOM='.dochar('rien'), 'TEST_ETAT='.dochar('oui') );
is ( $res, 4, '__del_par_champ' );

# /5-6
$res = __par_clef($dbh,'TEST','rien','TEST_NOM');
is ( ref($res), 'HASH', '__del_par_champ');
is ( $res->{'TEST_ID'}, 4, '__del_par_champ');

# /7
__del_par_champ($dbh, 'TEST', 'NOM', dochar('rien') );
$res = __par_clef($dbh,'TEST',dochar('rien'),'TEST_NOM');
is ( $res, undef, '__del_par_champ');

# /8
$res = [ __get_relation($dbh,'A','B','A',4) ];
is ( $#$res, 0, '__del_relation' );

# /9
__del_relation($dbh,'A','B',4,2);
$res = [ __get_relation($dbh,'A','B','A',4) ];
is ( $#$res, -1, '__del_relation' );

# /10
$res = __set_self_relation($dbh, 'TEST', 1, 3, 'NOM='.dochar('selfrel3') );
ok ( $res, '__del_relation' );

# /11
$res = [ __get_self_relation($dbh, 'TEST', 'TEST', 1) ];
is ( $#$res, 2, '__del_relation' );

# /12
__del_relation($dbh,'TEST','TEST',1,3,'NOM='.dochar('selfrel2'));
$res = [ __get_self_relation($dbh, 'TEST', 'TEST', 1) ];
is ( $#$res, 1, '__del_relation' );

# /13
$res = __set_rel_val($dbh,'AB',{'A_ID'=>3, 'B_ID'=>1, 'AB_NOM'=>dochar('type2')},
	'AB_NOM', dochar('type3'));
ok ( $res, '__set_rel_val' );

# /14-16
$res = __par_clef($dbh,'AB','type3','AB_NOM');
is ( ref($res), 'HASH', '__set_rel_val' );
is ( $res->{'A_ID'}, 3, '__set_rel_val' );
is ( $res->{'B_ID'}, 1, '__set_rel_val' );

# Before: 195 + (/16) -> 211

####
#### __rotate_table -> 15
####

# /1
__new($dbh,'LOG','LOG_INFO='.dochar('info  1'),'LOG_DATE='.dodate(20050401120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info  2'),'LOG_DATE='.dodate(20050402120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info  3'),'LOG_DATE='.dodate(20050403120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info  4'),'LOG_DATE='.dodate(20050404120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info  5'),'LOG_DATE='.dodate(20050405120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info  6'),'LOG_DATE='.dodate(20050406120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info  7'),'LOG_DATE='.dodate(20050407120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info  8'),'LOG_DATE='.dodate(20050408120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info  9'),'LOG_DATE='.dodate(20050409120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 10'),'LOG_DATE='.dodate(20050410120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 11'),'LOG_DATE='.dodate(20050415120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 12'),'LOG_DATE='.dodate(20050420120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 13'),'LOG_DATE='.dodate(20050425120000),"A_ID=4");
$res = [ __par_date($dbh,'LOG','LOG_DATE','an','20050000000000') ];
is ( $#$res, 12, '__par_date' );

# /2
$res = [ __par_date($dbh,'LOG','LOG_DATE','mois','20050400000000') ];
is ( $#$res, 12, '__par_date' );

# /3
$res = [ __par_date($dbh,'LOG','LOG_DATE','jour','20050405000000') ];
is ( $#$res, 0, '__par_date');

# /4
$res = [ __par_date($dbh,'LOG','LOG_DATE','jour','20050412121212') ];
is ( $#$res, -1, '__par_date');

# /5
$res = [ __par_date($dbh,'LOG','LOG_DATE','heure','20050410121212') ];
is ( $#$res, 0, '__par_date');

# /6
$res = [ __par_date($dbh,'LOG','LOG_DATE','heure','20050410131212') ];
is ( $#$res, -1, '__par_date');

# /7
$res = [ __par_date($dbh,'LOG','LOG_DATE','seconde','20050410120000') ];
is ( $#$res, 0, '__par_date');

# /8
$res = [ __par_date($dbh,'LOG','LOG_DATE','seconde','20050410120001') ];
is ( $#$res, -1, '__par_date');

# /9
$res = [ __par_date($dbh,'LOG','LOG_DATE','seconde','20050410115959') ];
is ( $#$res, -1, '__par_date');

# /10
$res = __rotate_table($dbh,'LOG','20050501020000');
is ( $res, 1, '__rotate_table' );

# /11
__new($dbh,'LOG','LOG_INFO='.dochar('info 14'),'LOG_DATE='.dodate(20050501120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 15'),'LOG_DATE='.dodate(20050503120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 16'),'LOG_DATE='.dodate(20050504120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 17'),'LOG_DATE='.dodate(20050506120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 18'),'LOG_DATE='.dodate(20050508120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 19'),'LOG_DATE='.dodate(20050509120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 20'),'LOG_DATE='.dodate(20050510120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 21'),'LOG_DATE='.dodate(20050512120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 22'),'LOG_DATE='.dodate(20050513120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 23'),'LOG_DATE='.dodate(20050514120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 24'),'LOG_DATE='.dodate(20050523120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 25'),'LOG_DATE='.dodate(20050525120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 26'),'LOG_DATE='.dodate(20050601010000),"A_ID=3");
$res = __rotate_table($dbh,'LOG','20050601020000');
is ( $res, 2, '__rotate_table' );

# /12
__new($dbh,'LOG','LOG_INFO='.dochar('info 27'),'LOG_DATE='.dodate(20050601120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 28'),'LOG_DATE='.dodate(20050604120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 29'),'LOG_DATE='.dodate(20050607120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 30'),'LOG_DATE='.dodate(20050609120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 31'),'LOG_DATE='.dodate(20050611120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 32'),'LOG_DATE='.dodate(20050612120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 33'),'LOG_DATE='.dodate(20050613120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 34'),'LOG_DATE='.dodate(20050615120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 35'),'LOG_DATE='.dodate(20050630120000),"A_ID=3");
$res = __rotate_table($dbh,'LOG','20050701020000');
is ( $res, 3, '__rotate_table' );

__new($dbh,'LOG','LOG_INFO='.dochar('info 36'),'LOG_DATE='.dodate(20050701120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 37'),'LOG_DATE='.dodate(20050709120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 38'),'LOG_DATE='.dodate(20050712120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 39'),'LOG_DATE='.dodate(20050714120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 40'),'LOG_DATE='.dodate(20050718120000),"A_ID=3");
__new($dbh,'LOG','LOG_INFO='.dochar('info 41'),'LOG_DATE='.dodate(20050722120000),"A_ID=3");

# /13
$res = __rotate_table($dbh,'NOLOG');
is ( $res, 4, '__rotate_table' );

# /14
$res = __par_id($dbh,'ROTATION',4);
is ( ref($res), 'HASH', '__rotate_table');

# /15
$res = FDN::Sql::DB_Access::drop_table($dbh,$res->{'ROTATION_DEST'});
ok ( $res->{'ok'}, '__rotate_table');
# Before: 211 + (/15) -> 226


####
#### __par_champ_ext -> 86
####

# /1-3
my $opts;
$test = \&FDN::Sql::Basic::__rewrite_range_option;
$opts = {};
$res = &$test('10-19',$opts);
ok ( $res,                  '__rewrite_range_option' );
is ( $opts->{'limit'},  10, '__rewrite_range_option' );
is ( $opts->{'offset'}, 10, '__rewrite_range_option' );

# /4-6
$opts = {};
$res = &$test('10+10',$opts);
ok ( $res,                  '__rewrite_range_option' );
is ( $opts->{'limit'},  10, '__rewrite_range_option' );
is ( $opts->{'offset'}, 10, '__rewrite_range_option' );

# /7-9
$opts = {};
$res = &$test('+10',$opts);
ok ( $res,                  '__rewrite_range_option' );
is ( $opts->{'limit'},  10, '__rewrite_range_option' );
is ( $opts->{'offset'},  0, '__rewrite_range_option' );

# /10-12
$opts = {};
$res = &$test('-9',$opts);
ok ( $res,                  '__rewrite_range_option' );
is ( $opts->{'limit'},  10, '__rewrite_range_option' );
is ( $opts->{'offset'},  0, '__rewrite_range_option' );

# /13-15
$opts = {};
$res = &$test('10+',$opts);
ok ( $res,                  '__rewrite_range_option' );
is ( $opts->{'limit'}, '18446744073709551615' , '__rewrite_range_option' );
is ( $opts->{'offset'}, 10, '__rewrite_range_option' );

# /16-18
$opts = {};
$res = &$test('10-',$opts);
ok ( $res,                  '__rewrite_range_option' );
is ( $opts->{'limit'}, '18446744073709551615', '__rewrite_range_option' );
is ( $opts->{'offset'}, 10, '__rewrite_range_option' );

# /19
ok ( 1, 'rien' );

# /20-24
$test = \&FDN::Sql::Basic::__rewrite_archives_option;
$opts = {};
$res = &$test('20050101000000>20050131230000',$opts);
ok ( $res,                                             '__rewrite_archives_option');
is ( $opts->{'datemin'},   '20050101000000',           '__rewrite_archives_option');
is ( $opts->{'datemax'},   '20050131230000',           '__rewrite_archives_option');
is ( $opts->{'datelast'},  'before',                   '__rewrite_archives_option');
is ( $opts->{'dateorder'}, 'orderdown(ROTATION_DATE)', '__rewrite_archives_option');

# /25-29
$opts = {};
$res = &$test('20050101000000<20050131230000',$opts);
ok ( $res,                                             '__rewrite_archives_option');
is ( $opts->{'datemin'},   '20050101000000',           '__rewrite_archives_option');
is ( $opts->{'datemax'},   '20050131230000',           '__rewrite_archives_option');
is ( $opts->{'datelast'},  'after',                    '__rewrite_archives_option');
is ( $opts->{'dateorder'}, 'orderup(ROTATION_DATE)',   '__rewrite_archives_option');

# /30-34
$test = \&FDN::Sql::Basic::__rewrite_archives_option;
$opts = {};
$res = &$test('20050101000000>20050131',$opts);
ok ( $res,                                             '__rewrite_archives_option');
is ( $opts->{'datemin'},   '20050101000000',           '__rewrite_archives_option');
is ( $opts->{'datemax'},   '20050131235959',           '__rewrite_archives_option');
is ( $opts->{'datelast'},  'before',                   '__rewrite_archives_option');
is ( $opts->{'dateorder'}, 'orderdown(ROTATION_DATE)', '__rewrite_archives_option');

# /35-39
$opts = {};
$res = &$test('20050101<20050131230000',$opts);
ok ( $res,                                             '__rewrite_archives_option');
is ( $opts->{'datemin'},   '20050101000000',           '__rewrite_archives_option');
is ( $opts->{'datemax'},   '20050131230000',           '__rewrite_archives_option');
is ( $opts->{'datelast'},  'after',                    '__rewrite_archives_option');
is ( $opts->{'dateorder'}, 'orderup(ROTATION_DATE)',   '__rewrite_archives_option');

# /40-44
$test = \&FDN::Sql::Basic::__rewrite_archives_option;
$opts = {};
$res = &$test('>20050131',$opts);
ok ( $res,                                             '__rewrite_archives_option');
is ( $opts->{'datemin'},   '0',                        '__rewrite_archives_option');
is ( $opts->{'datemax'},   '20050131235959',           '__rewrite_archives_option');
is ( $opts->{'datelast'},  'before',                   '__rewrite_archives_option');
is ( $opts->{'dateorder'}, 'orderdown(ROTATION_DATE)', '__rewrite_archives_option');

# /45-49
$opts = {};
$res = &$test('20050101<',$opts);
ok ( $res,                                            '__rewrite_archives_option');
is ( $opts->{'datemin'},   '20050101000000',           '__rewrite_archives_option');
is ( $opts->{'datemax'},   '0',                        '__rewrite_archives_option');
is ( $opts->{'datelast'},  'after',                    '__rewrite_archives_option');
is ( $opts->{'dateorder'}, 'orderup(ROTATION_DATE)',   '__rewrite_archives_option');

# /50
my $test2;
$test2 = \&FDN::Sql::Basic::__rewrite_option;
$opts = {};
$res  = &$test2('order','orderup(MACHIN)',$opts);
ok ( $res, '__rewrite_option' );

# /51
is_deeply ( $opts->{'order'}, [ 'orderup(MACHIN)' ], '__rewrite_option' );

# /52
$res  = &$test2('order', [ 'orderup(TRUC)' ], $opts );
ok ( $res, '__rewrite_option' );

# /53
is_deeply ( $opts->{'order'}, [ 'orderup(TRUC)' ], '__rewrite_option' );

# /54
$test2 = \&FDN::Sql::Basic::__find_archives_table;
$opts = { table => 'LOG', dbh => $dbh };
$res = &$test('20050501<20050601',$opts);
ok ( $res, '__find_archives_table' );

# /55
is ( $opts->{'datemin'},   '20050501000000',           '__rewrite_archives_option');

# /56
is ( $opts->{'datemax'},   '20050601235959',           '__rewrite_archives_option');

# /57
is ( $opts->{'datelast'},  'after',                    '__rewrite_archives_option');

# /58
is ( $opts->{'dateorder'}, 'orderup(ROTATION_DATE)',   '__rewrite_archives_option');

# /59
$res = &$test2($opts);
$exp = [	'LOG20050501020000',
		'LOG20050601020000',
		'LOG20050701020000' ];
is_deeply ( $opts->{'tables'}, $exp, '__find_archives_table' );

#  /60
$res = __par_champ_ext($dbh,'LOG_INFO',[],{'range' => '5+1','order'=>'orderup(LOG_ID)'});
is ( ref($res), 'HASH', '__par_champ_ext');

# /61
is ( $res->{'LOG_INFO'}, 'info 41', '__par_champ_ext');

# /62
$res = __par_champ_ext($dbh,'LOG_INFO',[],{'range' => '5+1','order'=>'orderup(LOG_ID)',
	'archives' => '20050702>' } );
is ( ref($res), 'HASH', '__par_champ_ext');

# /63
is ( $res->{'LOG_INFO'}, 'info 41', '__par_champ_ext');

# /64-65
$res = [ __par_champ_ext($dbh,'LOG',[],{
	'order'    => 'orderup(LOG_DATE)',
	'archives' => '20041201>20050425' } ) ];
is ( $#$res, 12, '__par_champ_ext' );
is ( $res->[12]->{'LOG_ID'}, 13, '__par_champ_ext');

# /66-71
$res = [ __par_champ_ext($dbh,'LOG', [
	[ 'LOG_DATE', '>=', '20050420000000' ],
	[ 'LOG_DATE', '<=', '20050615000000' ] ], {
	'order'    => 'orderup(LOG_DATE)',
	'archives' => '20050402000000<20050615000000' } ) ] ;
is ( $#$res, 21, '__par_champ_ext' );
is ( $res->[0]->{'LOG_ID'}, 12, '__par_champ_ext' );
is ( $res->[2]->{'LOG_ID'}, 14, '__par_champ_ext' );
is ( $res->[14]->{'LOG_ID'}, 26, '__par_champ_ext' );
is ( $res->[15]->{'LOG_ID'}, 27, '__par_champ_ext' );
is ( $res->[21]->{'LOG_ID'}, 33, '__par_champ_ext' );

# /72-74
$res = [ __par_champ_ext($dbh,'LOG', [
	[ 'LOG_DATE', '>=', '20050420000000' ],
	[ 'LOG_DATE', '<=', '20050615000000' ] ], {
	'order'    => 'orderup(LOG_DATE)',
	'archives' => '20050402000000<20050615000000',
	'range' => '20+0' }
	) ] ;
is ( $#$res, 1, '__par_champ_ext' );
is ( $res->[0]->{'LOG_ID'}, 32, '__par_champ_ext' );
is ( $res->[1]->{'LOG_ID'}, 33, '__par_champ_ext' );

# /75-77
$res = [ __par_champ_ext($dbh,'LOG', [
	[ 'LOG_DATE', '>=', '20050420000000' ],
	[ 'LOG_DATE', '<=', '20050615000000' ] ], {
	'order'    => 'orderup(LOG_DATE)',
	'archives' => '20050402000000<20050615000000',
	'range' => '10+7' }
	) ] ;
is ( $#$res, 6, '__par_champ_ext' );
is ( $res->[0]->{'LOG_ID'}, 22, '__par_champ_ext' );
is ( $res->[6]->{'LOG_ID'}, 28, '__par_champ_ext' );

# /78-80
$res = [ __par_champ_ext($dbh,'LOG', [
	[ 'LOG_DATE', '>=', '20050420000000' ],
	[ 'LOG_DATE', '<=', '20050615000000' ] ], {
	'order'    => 'orderdown(LOG_DATE)',
	'archives' => '20050402000000>20050615000000',
	'range' => '10+7' }
	) ] ;
ok ( $#$res >= 6, '__par_champ_ext' );
is ( $res->[0]->{'LOG_ID'}, 23, '__par_champ_ext' );
is ( $res->[6]->{'LOG_ID'}, 17, '__par_champ_ext' );

# /81-82
$res = [ __par_champ_ext( $dbh, 'LOG',
		[ [ 'LOG_DATE', '>=', '20050420000000' ],
		  [ 'LOG_DATE', '<=', '20050615000000' ],
		  [ 'LOG_INFO', '=',  dochar('info 13') ] ],
		{ 'archives' => '20050402000000>20050615000000' }
		)
       ] ;
ok ( $#$res == 0, '__par_champ_ext' );
is ( $res->[0]->{'LOG_ID'}, 13, '__par_champ_ext' );

# /83-84
$res = [ __par_champ_ext( $dbh, 'LOG',
		[ [ 'LOG_DATE', '>=', '20050420000000' ],
		  [ 'LOG_DATE', '<=', '20050615000000' ],
		  [ 'LOG.A_ID', '=', 'A.A_ID' ],
		  [ 'A.A_NOM', '=', dochar('titi') ] ],
		{ 'archives' => '20050402000000>20050615000000' }
		)
       ] ;
ok ( $#$res == 0, '__par_champ_ext' );
is ( $res->[0]->{'LOG_ID'}, 13, '__par_champ_ext' );

# /85-86
$res = __par_champ_test($dbh,'TEST',dochar('oui'),'TEST_ETAT','=');
is ( ref($res), 'HASH', '__par_champ_test');
is ( $res->{'TEST_ID'}, 1, '__par_champ_test');
# Before: 226 + (/82) -> 308

####
#### Pseudo-curseur: 9
####

my $rqh;
$exp = {
	3 => 'toto',
	4 => 'titi',
	5 => 'titi'
};

# /1
$rqh = __open_all($dbh,'A');
ok ( defined $rqh, '__open_all' );

# /2-3
$res = __read_all($rqh);
is ( ref($res), 'HASH', '__read_all');
is ( $res->{'A_NOM'}, $exp->{$res->{'A_ID'}}, '__read_all');

# /4-5
$res = __read_all($rqh);
is ( ref($res), 'HASH', '__read_all');
is ( $res->{'A_NOM'}, $exp->{$res->{'A_ID'}}, '__read_all');

# /6-7
$res = __read_all($rqh);
is ( ref($res), 'HASH', '__read_all');
is ( $res->{'A_NOM'}, $exp->{$res->{'A_ID'}}, '__read_all');

# /8
$res = __read_all($rqh);
is ( $res, undef, '__read_all');

# /9
__close_all($rqh);
ok (1, '__close_all');
# Before: 308 + (/9) -> 317



#FIXME
#__tous
